%% This class provides the GridController which connects a variable
%% number of houses and manages the battery as well as sale and purchase
%% process to the battery and the outside grid

classdef GridController < handle
   
    properties (Constant)
        % Path to block in our custom library
        SOURCE = 'smartlib/Microgrid_Controller';
        
        IN_CONS            = '/1';
        IN_PROD            = '/2';
        
        OUT_BATTERY_STATUS = '/1';
    end
    
    properties
        sys    %% Current Simulink file
        name   %% Name of Block in Simulink
        
        size   %% Holds the GridSize
        curr   %% Holds the current number of houses
    end
    
    methods
        
        %% Creates a new GridController which can handle
        %% a number of houses which is given by the gridSize
        function obj = GridController(system, gridSize)
            obj.size = gridSize;
            obj.curr = 0;
            
            inputs = ''; % Generates input string for ADD block
            for i = 1:gridSize, inputs = strcat(inputs, '+'); end
            
            obj.name = ['GridController' int2str(obj.getCount())];
            obj.sys = system.name; % Save target for later use
            
            % Adds the MicroGridController block to the file
            add_block(obj.SOURCE, [system.name '/' obj.name]);
            
            % Adds a block which adds all consumption outputs from the houses
            add_block(Simulink.ADD, [system.name '/' obj.name '_add_KWH_cons']);
            set_param([system.name '/' obj.name '_add_KWH_cons'],'Inputs', inputs);
            add_line(obj.sys, [obj.name '_add_KWH_cons/1'], [obj.name obj.IN_CONS]);
            
            % Adds a block which adds all production outputs from the houses
            add_block(Simulink.ADD, [system.name '/' obj.name '_add_KWH_prod']);
            set_param([system.name '/' obj.name '_add_KWH_prod'],'Inputs', inputs);
            add_line(obj.sys, [obj.name '_add_KWH_prod/1'], [obj.name obj.IN_PROD]);
            
        end
            
        %% Connects the taget block (which should be a house) with the 
        %% GridController. The name of the ports are auto generated
        function connect(obj, target)
            obj.curr = obj.curr + 1;
            
            add_line(obj.sys, [target.name target.OUT_KWH_CONS], [obj.name '_add_KWH_cons/' int2str(obj.curr)]);
            add_line(obj.sys, [target.name target.OUT_KWH_PROD], [obj.name '_add_KWH_prod/' int2str(obj.curr)]);
            
            add_line(obj.sys, [obj.name obj.OUT_BATTERY_STATUS], [target.name target.IN_BATTERY]);
        end
        
    end
    
    methods (Static)
        
        %% Returns the current number of GridControllers
        %% which are generated by the script
        function f = getCount()
             persistent anz;
             
             if isempty(anz), anz = 1; end
             if nargin, anz = anz + value; end

             f = anz;
        end
        
    end
    
end